%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% CLM test applied to Long-Run Risk Models of Nakamura, Steinsson, Barro adn Ursua(2013):
% Crises and Recoveries in an Empirical Model of Consumption Disasters
%
% September, 2017 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%  ReadMe
%
% 2. NoDisasters    = 0;         %0: case with disasters
%                                %1: case without disasters
% 
% 3. T = 742/12, it is the excercise for predictability of excess returns,
%      and this part is independendt with part 2.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
clear all;  clc; %close all;
addpath('.\Library\Library_RareDisaster_NSBU','.\Results\HJBounds','.\Results\replication\RepNSBU',...
       '.\Results\simulation','.\Results\simulation\SimNSBU','.\data','.\Library','.\Library.\Library_ConfidenceBands')
 
SET = 1;            % 1: SET A: stock market + risk free bond + 5-year government bonds
                    % 2: SET B: stock market + risk free bond + 5-,7-, 10-, 20-, 30-year government bonds
                    % 5: SET D: SET A + Value, Middle and Growth portfolios (new SET B in the revised version for JBES)
if SET == 1
%     load SDF_MH_SET1
    load SDF_MH_SET1_NewLong   % with E_m started from 0.3 for NSBU no Disasters case
    load CLM_BL_Bound_SET1
elseif SET == 5
    load SDF_MH_SET5
    load CLM_BL_Bound_SET5    
end

load Pred_Reg % estimation results from predictive regressions of asset returns
numPhi = size([beta_stock(1,:) beta_Tbill(1,:)],2);

horizon     = [1 5];      % 1-y and 5-year holding
Em          = E_m;             

T           = 742;   % 742

NSIM        = 100;
burnin      = 1000; %1000;


rand('seed', 1761)  
randn('seed', 1761)  

% rand('seed', 17234)  
% randn('seed', 17234)  

%% Initial Setting

global nSimDistDis maxDistDrop muPost sigmaEps  sigmaEta  meanPhi  sigmaPhi  meanTheta...
          sigmaTheta sigmaNu rhoZZ  p_W p_CbW p_CbI p_Ce
      
global nStateGridEps nStateGridZZ ZZshrinkFactor fracCutoffZZ
global onePerDis onePerPermDis nIntGridEps nIntGridEta nIntGridTheta nIntGridPhi nIntGridNu
global nSdEps  nSdEta nSdTheta nSdPhi nSdNu  

global  rhoU  gammaU  psiU  thetaU
global  payGrowth  pDefaultLong pDefaultShort debtEqRatio tolIntEq maxIter tolIntBond

% For getDistasterDist
NoDisasters    = 0;         %0: case with disasters
                            %1: case without disasters
% type of Disaster
onePerDis      = 0;
onePerPermDis  = 0;

tolIntEq       = 0.000001;   %Toleraunce for iterations on Equity integral
tolIntBond     = 0.000001;   % Tolerance for iterations on Bond integral
tolIntPerp     = 0.000001;   % Tolerance for iterations on Perpetuity integral
tolIntScaEq    = 0.000001;   % Tolerance for iterations on "scaled" Equity integral
tolIntOPEq     = 0.000001;   % Tolerance for iterations on One Period Equity integral

maxIter        = 1500;         % Maximum number of iterations

% Utility parameters
if onePerPermDis == 0
    gammaU     = 6.4;
else
    gammaU     = 3;
end
psiU           = 2;                 
rhoU           = 0.034;           
beta           = exp(-0.034);
thetaU         = (1-gammaU)/(1-1/psiU);

% Payoff properties of long term bond
payGrowth      = 0.90;      % Gross growth rate of payoff on the perpetuity
pDefaultLong   = 0.40;      % Probability that the perpetuity will suffer a partial default in each disaster period
pDefaultShort  = 0.00;      % Probability that the one period bond will suffer a partial default in each disaster period

% Leverage on leveraged equity
debtEqRatio    = 0.50;
PDivVecEqOld   = log(12);

% For getDistasterDist
nSimDistDis    = 1000;
useBaselinePar = 1;                % Use estimated parameters as opposed to, e.g., OnePerDis parameters
maxDistDrop    = 1.0;            % max percentage drop in PeakTrough and LongRun

% State Gridsize
nStateGridEps  = 1;
nStateGridZZ   = 601;
nStateGrid     =  nStateGridEps*nStateGridZZ*2;

% Integration Gridsize
nIntGridEps    = 1;
nIntGridEta    = 9;
nIntGridTheta  = 31;
nIntGridPhi    = 31;
nIntGridNu     = 1;
nIntGrid       = nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi*nIntGridNu;

% Grid limits in units of standard deviations
nSdEps         = 2;
nSdEta         = 3.5;
nSdTheta       = 3.5;
nSdPhi         = 3.5;
nSdNu          = 2;
fracCutoffZZ   = 0.025;

% ZZ Grid parameters
ZZshrinkFactor = 0.97;

% Parameters for simulating a "typical disaster"
nPrePeriods    = 5;
nDisPeriods    = 6;
nPostPeriods   = 25;

%% Get Parameter Estimates (we take the mean of posterior estimated parameters from paper directly, Table 1)
meanPhi        = -0.111;   sigmeanPhi     = 0.008;    % theta: a one-off permanent shift in the level of potential consumption, x_t
meanTheta      = -0.025;   sigmeanTheta = 0.007;      % phi: a shock that causes a temporary drop in consumption due to the disaster at time t

p_W            = 0.037;    sigp_W        = 0.016;     % probability of a world-wide disaster 
p_CbW          = 0.623;    sigp_CbW      = 0.076;     % probability that a particular country will enter a disaster when a world disaster begins
p_CbI          = 0.006;    sigp_CbI      = 0.003;     % probability that a particular country will enter a disaster on its own?
p_Ce           = 1-0.835;                             % probability to exit the disaster
p_CeC          = 1-p_Ce;   sigp_CeC      = 0.027;

rhoZZ          = 0.5;      sigrho_ZZ     = 0.034;     % firts order autoregressive coefficient of z_t

sigmaPhi       = 0.083;    sigsigmaPhi   = 0.006;
sigmaTheta     = 0.121;    sigsigmaTheta = 0.015;

% US country-specific estimates of parameters. See Table 3 and Table 4.
muPost         = 0.022;    sigmu         = 0.003;   % mean growth rate of potential consumption, post-1973 
sigmaEta       = 0.018;    sigsigmaEta   = 0.002;
sigmaEps       = 0.003;    sigsigmaEps   = 0.002;  % post-1946
sigmaNu        = 1e-5;       % this one is introduced to ad the convergence of the numberical algorithm
                              % set unimportant parameters to samll value, see in paper and their code

%% 
%% MODEL SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Load the previous simulation results to get mu_vFIXED
if NoDisasters  == 0 
load sim_NSBU2013_NSIM10_T150000
else
load sim_NSBU2013_NoDisasters_NSIM10_T150000
end
mu_vFIXED = mean(simmean_c,1);


%% Draw all the parameters from a normal distribution
Beta       = [meanPhi meanTheta p_W p_CbW p_CbI p_CeC rhoZZ sigmaPhi sigmaTheta...
                     muPost sigmaEta sigmaEps];
numBeta    = size(Beta,2);
SIG_Beta= diag([sigmeanPhi sigmeanTheta sigp_W sigp_CbW sigp_CbI sigp_CeC sigrho_ZZ...
                              sigsigmaPhi sigsigmaTheta sigmu sigsigmaEta sigsigmaEps].^2);
randnNum    = randn(NSIM,numBeta);%zeros(NSIM,numBeta);
for ii = 1:numBeta
    Beta_sim(:,ii) = Beta(ii) + randnNum(:,ii).*sqrt(SIG_Beta(ii,ii))/5;
end

%% Simulation 
if T == 742     % cut the small sample from a long simulation 
    J = 750; %100;300;750(best);6250(best for mixed mean);
    K = 184+J;
    G = 0;  %2000
else
    J = 1;
    K = TSIM;
    G = 0;
end

for sim =1:NSIM
    sim
    meanPhi = Beta_sim(sim,1);
    meanTheta = Beta_sim(sim,2);
    if Beta_sim(sim,3) <0 | Beta_sim(sim,3) >0.5
        p_W = Beta(3);
    else
        p_W = Beta_sim(sim,3);
    end
    if Beta_sim(sim,4) <0 | Beta_sim(sim,4) <0.5
        p_CbW = Beta(4);
    else
        p_CbW = Beta_sim(sim,4);
    end
    if Beta_sim(sim,5) <0 | Beta_sim(sim,5) >0.5
        p_CbI = Beta(5);
    else
        p_CbI = Beta_sim(sim,5);
    end    
    if Beta_sim(sim,6) <0 | Beta_sim(sim,6) >0.5
        p_CeC = Beta(6);
    else
        p_CeC = Beta_sim(sim,6);
    end
    p_Ce = 1-p_CeC;
    if Beta_sim(sim,7)<1
        rhoZZ = Beta_sim(sim,7);
    else
        rhoZZ = Beta(7);
    end
    sigmaPhi = Beta_sim(sim,8);
    sigmaTheta = Beta_sim(sim,9);
    muPost = Beta_sim(sim,10);
    sigmaEta = Beta_sim(sim,11);
    sigmaEps = Beta_sim(sim,12);

    
    %% Generate Disasters and create grids
    [DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ,DistXX,...
        DistZZ, DistCC, medianCC, q25CC, q75CC] =DistDisasters(nSimDistDis,maxDistDrop);
    
    % Create State Grid
    [stateGrid,  stateGridII, stateGridEps, stateGridZZ]   = createStateGrid(DistMinZZ,DistMaxZZ);
    
    % Create Integration grid
    [intGrid, intGridEps, intGridEta, intGridTheta, intGridPhi, intGridNu, tempGridEps, tempGridTheta, tempGridPhi, ...
        tempGridEta, tempGridNu ] = createIntGrid(DistDropLongRun, DistDropPeakTrough);
    
    % Calculate weights over IntGrid
    weightsOverIntGrid = calcWeights(intGridEps,  intGridEta, intGridTheta, intGridPhi, intGridNu,...
        tempGridEps, tempGridTheta, tempGridPhi, tempGridEta, tempGridNu,...
        DistDropLongRun, DistDropPeakTrough);
    
    %% Solve PD ratio
    %  Generate nextStates for both disaster and no disaster cases
%     nextStateFuncGen (stateGrid, stateGridEps, stateGridZZ, intGrid, intGridEps, intGridEta, ...
%         intGridTheta, intGridPhi, intGridNu,1);
%     %  Solving PD ratio for both equity and one period bond(bill)
%     
%%   for each simulation we have to regenerate the grids of logPD ratio, you have to run these line for the first time
%%   and all the results would be saved in the simulation folder.
%     initPDivEq = PDivVecEqOld;
%     [PDivVecEq, PDivVecOnePerBond] = PDivs_EZW (stateGrid,intGrid, weightsOverIntGrid,initPDivEq);
%     fname = [cd, '\Results\simulation\SimNSBU\logPDivRatio_NSIM',num2str(sim)];
%     save(fname,'PDivVecEq','PDivVecOnePerBond')
    
    logPDivRatio= [cd, '\Results\simulation\SimNSBU\logPDivRatio_NSIM',num2str(sim)];
%     logPDivRatio= [cd, '\Results\simulation\SimNSBU\logPDivRatio_ii12_jj1'];
    load(logPDivRatio);
    
    PDivVecEqOld = PDivVecEq;
    
    %% construct state variables
    
    % Set initial values
    II       =  zeros(TSIM+burnin,1);
    IIWorld  =  zeros(TSIM+burnin,1);
    XX       =  zeros(TSIM+burnin,1);
    ZZ       =  zeros(TSIM+burnin,1);
    Eps      =  zeros(TSIM+burnin,1);
    CC       =  zeros(TSIM+burnin,1);
    Eta      =  zeros(TSIM+burnin,1);
    Phi      =  zeros(TSIM+burnin,1);
    Theta    =  zeros(TSIM+burnin,1);
    Nu       =  zeros(TSIM+burnin,1);
    
    XX(1)    = 1;
    CC(1)    = XX(1) + ZZ(1) + Eps(1);
    
    %% Simulate model
    for tt = 2: TSIM+burnin
        IIWorld(tt)     = binornd(1,p_W);
        
        if  II(tt-1) == 1
            temppp = p_CeC;      %ppC(2)
        else
            if IIWorld(tt) == 1
                temppp = p_CbW;  %ppC(3)
            else
                temppp = p_CbI;  %ppC(1)
            end
        end
        
        if NoDisasters ~=0
            II(tt)              = 0;
        else
            II(tt)              = binornd(1,temppp);
        end
        
        Theta(tt)    = meanTheta+sigmaTheta*randn(1);
        

        k = 1;
        while k < 2
            rdnumber = meanPhi + sigmaPhi*randn(1,1);
            if rdnumber < 0 & rdnumber> -1000
                Phi(tt) = rdnumber; % truncated normal variable
                k = k +1;
            else
                k = k;
            end
        end
        
        shock     = randn(1,3);
        Eps(tt)   = sigmaEps*shock(1);
        Eta(tt)   = sigmaEta*shock(2);
        Nu(tt)    = sigmaNu *shock(3);
        
        XX(tt)    = XX(tt-1) + muPost + Eta(tt) + Theta(tt)*II(tt);
        ZZ(tt)    = rhoZZ*ZZ(tt-1) + Nu(tt) - Theta(tt)*II(tt) + Phi(tt)*II(tt);
        CC(tt)    = XX(tt) + ZZ(tt) + Eps(tt);
    end
    
    sim_IIWorld      = IIWorld((burnin+1):TSIM+burnin);
    sim_II           = II((burnin+1):TSIM+burnin);
    sim_Eps          = Eps((burnin+1):TSIM+burnin);
    sim_Eta          = Eta((burnin+1):TSIM+burnin);
    sim_Theta        = Theta((burnin+1):TSIM+burnin);
    sim_Phi          = Phi((burnin+1):TSIM+burnin);
    sim_Nu           = Nu((burnin+1):TSIM+burnin);
    sim_XX           = XX((burnin+1):TSIM+burnin);
    sim_ZZ           = ZZ((burnin+1):TSIM+burnin);
    sim_CC           = CC((burnin+1):TSIM+burnin);
    
    % put simulation output on state grid for computing consumption wealth returns
    [sim_EpsOnGrid, sim_ZZOnGrid] = SimOutputOnStateGrid(sim_Eps, sim_ZZ, stateGridEps, stateGridZZ);
    
    % calculate return on equity(or return based on consumption claims) using simulated data from model
    [Return, CGrowth, PDEquity]    = calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecEq, stateGridII, stateGridEps, stateGridZZ, 0);
    [Rf, CGrowth, PDBond]          = calcReturns(sim_CC, sim_EpsOnGrid, sim_ZZOnGrid, sim_II,  PDivVecOnePerBond, stateGridII, stateGridEps, stateGridZZ, 1);

    %% Construct 1-year and 5-year MRS from annualy data
    MRS_1Y               = exp( thetaU*log(beta) + (-thetaU/psiU)*log(CGrowth(2:end)) + (-(1-thetaU))*log(Return));
    MRS_5Y               = exp(chgfreq(log(MRS_1Y),horizon(2),horizon(2),horizon(2)-1));

%%    
    if T == 742
        sim_MRS_1Y(:,sim) = MRS_1Y(J:K,1);
        sim_MRS_5Y(:,sim) = MRS_5Y(J:K,1);
        sim_SDF_1Y(:,sim) = MRS_1Y;
        sim_SDF_5Y(:,sim) = MRS_5Y;
        sim_II_save(:,sim)=II(1001+J:K+1001,1);
    else
        sim_MRS_1Y(:,sim) = MRS_1Y;
        sim_MRS_5Y(:,sim) = MRS_5Y;
    end
  
    % Compute the mean, mean[SDF]
    simmean_1Y(sim,1)   = mean(sim_MRS_1Y(:,sim),1);    
    simmean_5Y(sim,1)   = mean(sim_MRS_5Y(:,sim),1);
    
    %% [Random Random] 
    % Compute the standard deviation of simulated SDF, std(SDF)
    simsd_1Y(sim,1)     = std(sim_MRS_1Y(:,sim),1);
    simsd_5Y(sim,1)     = std(sim_MRS_5Y(:,sim),1);
    % Compute the distance between simulated SDF and simulated BL bound
    [TF1Y  diff1Y_RandRand(sim,1) sigbound1Y_RandRand(sim,1)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,sim+G)),simmean_1Y(sim,1),simsd_1Y(sim,1));
    [TF5Y  diff5Y_RandRand(sim,1) sigbound5Y_RandRand(sim,1)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,sim+G)),simmean_5Y(sim,1),simsd_5Y(sim,1));

    %% [Random Fixed] 
    % Compute the distance between simulated SDF and estimated BL bound
    [TF1Y  diff1Y_RandFixed(sim,1) sigbound1Y_RandFixed(sim,1)]  = IsItWithinHJB(Em,sqrt(sig2_BL(2,:)),simmean_1Y(sim,1),simsd_1Y(sim,1));
    [TF5Y  diff5Y_RandFixed(sim,1) sigbound5Y_RandFixed(sim,1)]  = IsItWithinHJB(Em,sqrt(sig2_BL(3,:)),simmean_5Y(sim,1),simsd_5Y(sim,1));
    
    %% [Fixed Random] 
    % Compute the standard deviation of SDF, when mu_v is fixed, SDF-Mean(SDF)+mu_vFixed and take its STDEV
    MRS_1Y_vFIXED       = sim_MRS_1Y(:,sim) - (simmean_1Y(sim,1) - mu_vFIXED(1))*ones(size(sim_MRS_1Y(:,sim)));
    MRS_5Y_vFIXED       = sim_MRS_5Y(:,sim) - (simmean_5Y(sim,1) - mu_vFIXED(2))*ones(size(sim_MRS_5Y(:,sim)));
    simmean_1Y_vFIXED(sim,1)   = mean(MRS_1Y_vFIXED,1);
    simmean_5Y_vFIXED(sim,1)   = mean(MRS_5Y_vFIXED,1);
    simsd_1Y_vFIXED(sim,1)     = std(MRS_1Y_vFIXED,1);
    simsd_5Y_vFIXED(sim,1)     = std(MRS_5Y_vFIXED,1);
    % % Compute the distance between simulated SDF and estimated BL bound when mu_v is fixed
    [TF1Y  diff1Y_FixedRand(sim,1) sigbound1Y_FixedRand(sim,1)]  = IsItWithinHJB(sim_E_m1Y,sqrt(sim_sig2_BL1Y(:,sim+G)),simmean_1Y_vFIXED(sim,1),simsd_1Y_vFIXED(sim,1));
    [TF5Y  diff5Y_FixedRand(sim,1) sigbound5Y_FixedRand(sim,1)]  = IsItWithinHJB(sim_E_m5Y,sqrt(sim_sig2_BL5Y(:,sim+G)),simmean_5Y_vFIXED(sim,1),simsd_5Y_vFIXED(sim,1));    

end

diff_RandRand      = [diff1Y_RandRand diff5Y_RandRand];
diff_RandFixed     = [diff1Y_RandFixed diff5Y_RandFixed];
diff_FixedRand     = [diff1Y_FixedRand diff5Y_FixedRand];

%% Find NaN Values from diff (see IsItWithinHJB.m)
[row_RandRand col_RandRand] = find(isnan(diff_RandRand));
[row_RandFixed col_RandFixed] = find(isnan(diff_RandFixed));
[row_FixedRand col_FixedRand] = find(isnan(diff_FixedRand));

%% Report and Save the Results
mean_diff_RandRand = nanmean(diff_RandRand,1);
sd_diff_RandRand   = nanstd(diff_RandRand,1);
temp = sort(diff_RandRand);
a = temp(:,1); b = temp(:,2);
qut_diff_RandRand  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])'];
prob_RandRand = [size(find(diff_RandRand(:,1)<0),1) size(find(diff_RandRand(:,2)<0),1)]./size(diff_RandRand,1);

mean_diff_RandFixed = nanmean(diff_RandFixed,1);
sd_diff_RandFixed   = nanstd(diff_RandFixed,1);
temp = sort(diff_RandFixed);
a = temp(:,1); b = temp(:,2); 
qut_diff_RandFixed  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])'];
prob_RandFixed = [size(find(diff_RandFixed(:,1)<0),1) size(find(diff_RandFixed(:,2)<0),1)]./size(diff_RandFixed,1);

mean_diff_FixedRand = nanmean(diff_FixedRand,1);
sd_diff_FixedRand   = nanstd(diff_FixedRand,1);
temp = sort(diff_FixedRand);
a = temp(:,1); b = temp(:,2); 
qut_diff_FixedRand  = [quantile(a,[0.05 0.5 0.95])' quantile(b,[0.05 0.5 0.95])'];
prob_FixedRand = [size(find(diff_FixedRand(:,1)<0),1) size(find(diff_FixedRand(:,2)<0),1)]./size(diff_FixedRand,1);

%% Reported Values in Table 3
[qut_diff_FixedRand(3,:)' qut_diff_RandFixed(3,:)' qut_diff_RandRand(3,:)' ]

%% Save the results
if NoDisasters == 0
    fname = [cd, '\Results\simulation\CLM_RaraDisaster_NSBU_NSIM',num2str(NSIM),'_T',num2str(T),'_SET',num2str(SET)];
else
    fname = [cd, '\Results\simulation\CLM_RaraDisaster_NSBU_NoDisasters_NSIM',num2str(NSIM),'_T',num2str(T),'_SET',num2str(SET)];
end
save(fname,'diff_RandRand','diff_RandFixed','diff_FixedRand','sim_MRS_1Y','sim_MRS_5Y');

