%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Multi-horizon Stochastic Discount Factor Bounds 
%
% Implications of Return Predictability for Consumpition Dyanmiac and Asset Pricing
% by  Carlo Favero, Fulvio Ortu, Andrea Tamoni and Haoxi Yang
%
% June, 2015   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This code compute the multi-horizon SDF bounds with v.s. without
% predictive information
% -------------------------------------------------------------------------
% SET 1: stock market + 3 month treasury government bill [roll-over strategy]
% 
% SET 5: SET A + Value Growth portfolios (new SET B in the revised version for JBES)
% -------------------------------------------------------------------------
% 1. We only use real returns for all the assets in this excercise
% 2. This exercise studies 1-quarter 1-year and 5-year SDF bounds properties
% -------------------------------------------------------------------------
% 
%% Initial Setting
clear all; clc; close all;
addpath('.\Library\gmm;','.\Results\HJBounds','.\Library','.\data',...
    '.\Library\minz;','.\Library\HJBLibrary','.\Library\Library_CHJ')
addpath('.\Results\simulation','.\Results\HJBounds')

SET = 5;               % 1: SET A
                       % 5: SET E (new SET B in the revised version  for JBES, with also middle
                              % portfolio from classical and industry adjusted portfoilios)
load longrun5Portfolios.mat   % 1947Q2-2012Q4
cstr=cal(1947,2,4,255); 
ss = ical(1952,2,cstr);       % sample size
se = ical(2012,3,cstr);

rmkt = r(ss:se,6); rPtf = r(ss:se,1:5); % real log return, 
% rPtf is for Fama-French 5-portfolio formed by BM ratio 
% [growth portfolio with low BM(return) and value portfolio with high BM(return)]
Rmkt=exp(rmkt);   RPtf = exp(rPtf);  Rrf =exp(rrf(ss:se));     % real gross return

lpd = log(QPD(ss+1:se+1,6));
InflationLong = Inflation(1:end-1); Inflation = Inflation(ss:se);

load  cay_q_12Q3.txt
cay = cay_q_12Q3(2:end,5);

load Pred_stock;
load Pred_bond;

% nominal value-growth portfolio returns and return predictors (See folder EqPtfsForFOTY)
load EqPtfsForFOTY; % norminal portfolio returns and return predictors
R_PtfReal = R_Ptf - repmat(Inflation,1,size(R_Ptf,2));

%% Fama-Bliss bond yields - using for constructing spread
% load bond price data
% this is the file from CRSP completely unmodified
% sample: 1952-06 - 2012-12
% We compute 1-year holding period return with different maturity FB discount bond

load Fama_Bliss_price.txt
bondprice = Fama_Bliss_price;
T=length(bondprice); %(2012-1952+1)*12 -5 -3 

y=-log(bondprice(:,2:end)/100).*(ones(T,1)*[1/1 1/2 1/3 1/4 1/5]);
famablisyld=[bondprice(:,1) y];
yields=famablisyld(:,2:end);
mats=[1 2 3 4 5]'; 
prices=-(ones(T,1)*mats').*yields;

forwards = prices(:,1:4)-prices(:,2:5);
fs = forwards-yields(:,1)*ones(1,4);      % forward spread 

hpr = prices(13:T,1:4)-prices(1:T-12,2:5);% holding period returns
hprx =  hpr - yields(1:T-12,1)*ones(1,4);
hpr  = [zeros(12,1)*ones(1,4); hpr];      % pads out the initial values with zeros so same length as other series       
hprx = [zeros(12,1)*ones(1,4); hprx];


hprQ   = hpr(1:3:end-1,:);
hprxQ  = hprx(1:3:end-1,:);
yieldsQ= yields(1:3:end-1,:);

keep Rmkt Rrf lpd Inflation SET InflationLong cay yields hprQ yieldsQ Pred_stock Pred_bond Pred_Ptf R_Ptf R_PtfReal

%% US Government Bonds 
%% loaded from Indices/Deciles: Treasury and Inflation -- US Treasury and Inflation - Monthly

load .\data\bondreturns19252012.txt;

cstr=cal(1925,12,12,size(bondreturns19252012,1)); 
ss  = ical(1952,6,cstr); se = ical(2012,12,cstr);

bondret30yQ=chgfreq(bondreturns19252012(:,2),3,1,0);
bondret30yQ=bondret30yQ(ss:3:se-3);

bondret20yQ=chgfreq(bondreturns19252012(:,3),3,1,0);
bondret20yQ=bondret20yQ(ss:3:se-3);

bondret10yQ=chgfreq(bondreturns19252012(:,4),3,1,0);
bondret10yQ=bondret10yQ(ss:3:se-3);

bondret7yQ=chgfreq(bondreturns19252012(:,5),3,1,0);
bondret7yQ=bondret7yQ(ss:3:se-3);

bondret5yQ=chgfreq(bondreturns19252012(:,6),3,1,0);
bondret5yQ=bondret5yQ(ss:3:se-3);

bondret2yQ=chgfreq(bondreturns19252012(:,7),3,1,0);
bondret2yQ=bondret2yQ(ss:3:se-3);

bondret1yQ=chgfreq(bondreturns19252012(:,8),3,1,0);
bondret1yQ=bondret1yQ(ss:3:se-3);

% the spread is the difference between the log yield on a 5-year artificial zero-coupon bond from the CRSP Fama-Bliss Discount Bond File, and the log yield on the Treasury Bill
TB1M = bondreturns19252012(ss:se,10); 
spr  = yields(:,5)/12-TB1M; sprQ=spr(1:3:end-3);

TB1M = chgfreq(bondreturns19252012(:,10),3,1,0); %The short rate is the log yield on the 30-day Treasury Bill from CRSP
TB1M = TB1M(ss:3:se-3);

% Roll over 3M Tbill returns
TB3MSretQ = chgfreq(bondreturns19252012(:,9),3,1,0);
TB3MSretQ = TB3MSretQ(1:3:end-3);

cstr=cal(1925,4,4,size(TB3MSretQ,1)); 
ss  = ical(1947,2,cstr); se = ical(2012,3,cstr);
TB3MSretQ = TB3MSretQ(ss:se);  

% realized roll over 3M Tbill returns
TB3MSretQReal   = TB3MSretQ - InflationLong;
TB3MSroll1YReal = chgfreq(TB3MSretQReal,4,1,0);
TB3MSroll5YReal = chgfreq(TB3MSretQReal,4*5,1,0);     %THIS IS NOT ANNUALIZED

cstr=cal(1947,2,4,size(TB3MSretQ,1)); 
ss  = ical(1952,2,cstr);       
se  = ical(2012,3,cstr);
TB3MrollReal = [TB3MSretQReal(ss:se) TB3MSroll1YReal(ss:se) TB3MSroll5YReal(ss:se)];

% realized rate of government bonds with different maturities
bondret1yQ_real  = 1 + bondret1yQ -Inflation;
bondret2yQ_real  = 1 + bondret2yQ -Inflation;
bondret5yQ_real  = 1 + bondret5yQ -Inflation;
bondret7yQ_real  = 1 + bondret7yQ -Inflation;
bondret10yQ_real = 1 + bondret10yQ-Inflation;
bondret20yQ_real = 1 + bondret20yQ-Inflation;
bondret30yQ_real = 1 + bondret30yQ-Inflation;

%% Predictive Regressions of Returns
horizons = [1 4 5*4];        % 1-quarter, 1-year and 5-year


if  SET   == 1
    assetret = NaN(size(Rmkt,1),2,size(horizons,2));
elseif SET == 5
    assetret = NaN(size(Rmkt,1),10,size(horizons,2));    
end
E_m          = 0.3:0.0005:2.5;

sig2_FS  = NaN(length(horizons),length(E_m)); sig2_BL         = NaN(length(horizons),length(E_m)); sig2_naive = NaN(length(horizons),length(E_m));
sig2_GHT = NaN(length(horizons),length(E_m)); sig2_BL_stacked = NaN(length(horizons),length(E_m));
for hh=1:length(horizons)
    horizon = horizons(hh); frequency = 1; offset = 0;
    y1     = exp(trimr(chgfreq(log(Rmkt),horizon,frequency,offset),horizon,0));
    y2     = exp(trimr(TB3MrollReal(:,hh),horizon,0));
        
    yb1y   = exp(trimr(chgfreq(log(bondret1yQ_real) ,horizon,frequency,offset),horizon,0));
    yb2y   = exp(trimr(chgfreq(log(bondret2yQ_real) ,horizon,frequency,offset),horizon,0));
    yb5y   = exp(trimr(chgfreq(log(bondret5yQ_real) ,horizon,frequency,offset),horizon,0));
    yb7y   = exp(trimr(chgfreq(log(bondret7yQ_real) ,horizon,frequency,offset),horizon,0));
    yb10y  = exp(trimr(chgfreq(log(bondret10yQ_real),horizon,frequency,offset),horizon,0));
    yb20y  = exp(trimr(chgfreq(log(bondret20yQ_real),horizon,frequency,offset),horizon,0));
    yb30y  = exp(trimr(chgfreq(log(bondret30yQ_real),horizon,frequency,offset),horizon,0));
     
    % Value-Growth portfolio returns
    yp1    = exp(trimr(chgfreq(log(R_PtfReal(:,1)),horizon,frequency,offset),horizon,0));
    yp2    = exp(trimr(chgfreq(log(R_PtfReal(:,2)),horizon,frequency,offset),horizon,0));
    yp3    = exp(trimr(chgfreq(log(R_PtfReal(:,3)),horizon,frequency,offset),horizon,0));
    yp4    = exp(trimr(chgfreq(log(R_PtfReal(:,4)),horizon,frequency,offset),horizon,0));
    yp5    = exp(trimr(chgfreq(log(R_PtfReal(:,5)),horizon,frequency,offset),horizon,0));
    yp6    = exp(trimr(chgfreq(log(R_PtfReal(:,6)),horizon,frequency,offset),horizon,0));
    yp7    = exp(trimr(chgfreq(log(R_PtfReal(:,7)),horizon,frequency,offset),horizon,0));
    yp8    = exp(trimr(chgfreq(log(R_PtfReal(:,8)),horizon,frequency,offset),horizon,0));
    %% %%%%%%%%%%%%%%%%%%%%%%%%

    
    smpl_range = size(y1,1);    
    x1    = [ones(smpl_range,1) trimr(Pred_stock,0,horizon)];
    xb    = [ones(smpl_range,1) trimr(Pred_bond,0,horizon)];
    % ---------------------------------------------------------------------
    % regressors of Value Growth Portfolios
    xp1   = [ones(smpl_range,1) trimr(Pred_Ptf(:,1),0,horizon) trimr(Pred_stock,0,horizon)];
    xp2   = [ones(smpl_range,1) trimr(Pred_Ptf(:,2),0,horizon) trimr(Pred_stock,0,horizon)];
    xp3   = [ones(smpl_range,1) trimr(Pred_Ptf(:,3),0,horizon) trimr(Pred_stock,0,horizon)];
    xp4   = [ones(smpl_range,1) trimr(Pred_Ptf(:,4),0,horizon) trimr(Pred_stock,0,horizon)];
    xp5   = [ones(smpl_range,1) trimr(Pred_Ptf(:,5),0,horizon) trimr(Pred_stock,0,horizon)];
    xp6   = [ones(smpl_range,1) trimr(Pred_Ptf(:,6),0,horizon) trimr(Pred_stock,0,horizon)];
    xp7   = [ones(smpl_range,1) trimr(Pred_Ptf(:,7),0,horizon) trimr(Pred_stock,0,horizon)];
    xp8   = [ones(smpl_range,1) trimr(Pred_Ptf(:,8),0,horizon) trimr(Pred_stock,0,horizon)];    

    %%  seemingly unrelated regression estimates    
    if SET == 1
        neqs = 2;
        y(1).eq = y1;    x(1).eq= x1;
        y(2).eq = y2;    x(2).eq= xb;
        assetret(1+horizon:end,:,hh) = [y1 y2];
    elseif SET == 5
        neqs = 10;
        y(1).eq = y1;    x(1).eq = x1;
        y(2).eq = y2;    x(2).eq = xb;
        y(3).eq = yp1;   x(3).eq = xp1;
        y(4).eq = yp2;   x(4).eq = xp2;
        y(5).eq = yp3;   x(5).eq = xp3;
        y(6).eq = yp4;   x(6).eq = xp4;
        y(7).eq = yp5;   x(7).eq = xp5;
        y(8).eq = yp6;   x(8).eq = xp6;
        y(9).eq = yp7;   x(9).eq = xp7;
        y(10).eq = yp8;   x(10).eq = xp8;
        assetret(1+horizon:end,:,hh) = [y1 y2 yp1 yp2 yp3 yp4 yp5 yp6 yp7 yp8];  
    end
    result = sur(neqs,y,x);
    
    %% keep estimated return and residuals
    if SET == 1
        mu_ret = [result(1).yhat  result(2).yhat  ];
        resid  = [result(1).resid result(2).resid ];
    elseif SET == 5
        mu_ret = [result(1).yhat  result(2).yhat result(3).yhat result(4).yhat result(5).yhat result(6).yhat result(7).yhat result(8).yhat result(9).yhat result(10).yhat];
        resid  = [result(1).resid result(2).resid result(3).resid result(4).resid result(5).resid result(6).resid result(7).resid result(8).resid result(9).resid result(10).resid];
    end
    sig2_ret   = result.sigma;
    
    %% Ferson and Siegel (2003) bounds
    mu_Pret       =-0.5:0.0005:3;            % setting expected portfolio returns
    if hh==length(horizons)
        target    = find(mu_Pret==2.9);      % location of an arbitrary portfolio expected returns
    else
        target    = find(mu_Pret==1.5);
    end
    sig2_FS(hh,:) = EFFICIENT_FS(E_m',mu_ret',sig2_ret,mu_Pret,target);
    
    %% GHT (1990) bounds with conditioning information
    sig2_GHT(hh,:) = OPTIMAL_GHT(E_m',mu_ret',sig2_ret);   
    
    %% Bekaert and Liu (2004) [optimal scaling bounds (OSB) and OSB stacked]
    sig2_BL(hh,:)         = OSB_BL_Robust( E_m',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);
    sig2_BL_stacked(hh,:) = OSB_BL_stacked(E_m',assetret(1+horizon:end,:,hh),mu_ret,sig2_ret);
end

%% HJ bounds without information
if SET == 3
    [mlop_1Y,output_1Y]=hj(assetret(horizons+1:end,:,1),E_m',[],0,0);
    sig2_HJ = var(mlop_1Y);
else
    [mlop_1Q,output_1Q]=hj(assetret(horizons(1)+1:end,:,1),E_m',[],0,0);
    sig2_HJ(1,:) = var(mlop_1Q);
    [mlop_1Y,output_1Y]=hj(assetret(horizons(2)+1:end,:,2),E_m',[],0,0);
    sig2_HJ(2,:) = var(mlop_1Y);
    [mlop_5Y,output_5Y]=hj(assetret(horizons(3)+1:end,:,3),E_m',[],0,0);
    sig2_HJ(3,:) = var(mlop_5Y);
end

%% Scaled variance bounds (naive bounds) [see Bekaert and Liu(2004), section 1.2]
% construct the system [r_t+1  r_t+1*predictors.]

for hh = 1:size(horizons,2)
    sys    = [assetret(:,:,hh) assetret(:,1,hh).*lpd assetret(:,1,hh).*cay];    
    for jj = 2 : size(assetret(:,:,hh),2)
        sys    = [sys assetret(:,jj,hh).*TB1M*100 assetret(:,jj,hh).*sprQ*100];
    end
    Ex     = mean(sys(horizons(hh)+1:end,:),1)';      %% mean
    cov_xx = cov(sys(horizons(hh)+1:end,:));          %% variance matrix
    zJ     = [lpd cay repmat([ TB1M*100 sprQ*100],1,size(assetret(:,:,hh),2)-1)];
    for i=1:length(E_m)                              %% number of points on HJ bound
        b = inv(cov_xx)*([ones(size(assetret(:,:,hh),2),1); mean(zJ)']-E_m(i)*Ex);
        sig2_naive(hh,i)=b'*cov_xx*b;                 %% Volatility of SDF
    end
    clear sys
end

%% Save the results
fname = [cd, '\Results\HJBounds\SDF_MH_SET',num2str(SET)];
if SET == 1
    save(fname, 'E_m','sig2_HJ','sig2_GHT','sig2_FS','sig2_BL','sig2_BL_stacked','sig2_naive','horizons');
 elseif SET == 5
     save(fname, 'E_m','sig2_HJ','sig2_GHT','sig2_FS','sig2_BL','sig2_BL_stacked','sig2_naive','horizons');    
end

