function  [TF, diff,sigbound]= IsItWithinHJB(Em,sigHJB, simmean,simsig)
% this is function is to justify whether the modole-based simulated SDFs
% located in the releated HJ bound.
% Input:  Em is the reasonable mean of SDF for each horizon (column vector)
%         sigHJB is the lower bound of realated horizon HJ bound with alpha
%         significance confidence interval
%         simmean, the mean of simulated SDFs
%         simsig, the standard deviation of simulated SDFs
% Output: TF   = 1 if it is in HJB
%              = 0 if not
%         diff = vertical distance from the model generated SDF and predicotor-based bound
%         sigbound = corresponding point on the predictor-based bound

if simmean > max(Em) | simmean < min(Em) | isnan(simmean)
    TF = 0;
    diff = NaN;
    sigbound = NaN;
else
    index = find (Em>=simmean);
    if simsig> 0.5*(sigHJB(index(1))+sigHJB(index(1)-1))
        TF = 1;
    else
        TF = 0;
    end
    diff = simsig-0.5*(sigHJB(index(1))+sigHJB(index(1)-1));
    sigbound = 0.5*(sigHJB(index(1))+sigHJB(index(1)-1));
end
