% Optimal Hansen-Jagannathan bounds [Gallant, Hanse and Tauchen (1990)]
%
% INPUT   Em    :  given unconditional mean of SDFs
%                  - scalar or vector
%         miu   :  conditional mean of assets returns  
%                  - N*T matrix
%                  - N: number of securities
%                  - T: number of observations
%         sig2   : conditional covariance matrix of assets returns 
% OUTPUT  sig2_m : unconditional variance of SDFs given Em
%
%         N      : # of assets
%         nobs   : # of observations
% function sig2_m = OPTIMAL_GHT(Em,miu,sig2)
function sig2_m = OPTIMAL_GHT(Em,miu,sig2,resid_R)

[N,nobs] = size(miu);
% replicate sig2 if it is constant case
if size(sig2,3) == 1
    sig2_R = repmat(sig2, [1 1 nobs]);
else
    sig2_R = sig2;
end
e = ones(N,1);
% sig2_R = cov(resid_R);
% alpha_Z = e'*inv(sig2_R)*e;
alpha_Z = zeros(nobs-1,1);
beta_Z  = zeros(nobs-1,1);
gamma_Z = zeros(nobs-1,1);
temp1   = zeros(nobs-1,1);
temp2   = zeros(nobs-1,1);
temp3   = zeros(nobs-1,1);

for ii =1:nobs-1
    alpha_Z(ii,1) = e'*inv(sig2_R(:,:,ii))*e;
    beta_Z(ii,1)  = e'*inv(sig2_R(:,:,ii))*miu(:,ii);
    gamma_Z(ii,1) = miu(:,ii)'*inv(sig2_R(:,:,ii))*miu(:,ii);
    temp1(ii,1)   = beta_Z(ii,1)/(1+gamma_Z(ii,1));
    temp2(ii,1)   = beta_Z(ii,1)^2/(1+gamma_Z(ii,1));
    temp3(ii,1)   = 1/(1+gamma_Z(ii,1));
end

sig2_m = NaN(1,length(Em));
for ii=1:length(Em)
    sig2_m(1,ii) = (Em(ii)-mean(temp1,1))^2/mean(temp3,1)+mean(alpha_Z)-mean(temp2,1)-Em(ii)^2;
end

if nargin==4
    zeta_Z  = zeros(nobs-1,length(Em));
    SDF_Z   = zeros(nobs-1,length(Em));
    for ii=1:length(Em)
        for jj =1:nobs-1
            zeta_Z(jj,ii)= temp1(jj,1)+temp3(jj,1)*((Em(ii)-mean(temp1,1))/mean(temp3,1));
            SDF_Z(jj,ii)= zeta_Z(jj,ii)+(e-zeta_Z(jj,ii)*miu(:,jj))'*inv(sig2_R(:,:,jj))*resid_R(jj,:)';
        end
    end
end