%Function fixed_point(p0, N) approximates the solution of an equation f(x) = 0, rewritten in the form x = g(x), which is a sub-function the user has to enter. the call to the function fixed_point(p0, N) returns the root of the equation f(x),i.e. the fixed-point of g(x), if the procedure is successful or a sequence of iterates in case something goes wrong. p0 is the initial approximation and N the maximum number of iterations. If, after N iterations, condition |x(k) - x(k-1)| < tol is not satisfied, all iterated values will be displayed, accompanied by a message asking the user to either change p0 in case of divergence or enter another g(x) that does not lead to complex numbers arithmetics. Reasons for the program to go wrong are the divergence of iterates and/or appearance of complex numbers for example with functions involving sqrt(x) when one of the iterates is negative. 
function pnew = fixed_point(p0, N)

%Fixed_Point(p0, N) approximates the root of the equation f(x) = 0
%rewritten in the form x = g(x), starting with an initial approximation p0. 
%The iterative technique is implemented N times.
%The user has to enter the function g(x) at the bottom


%Author: Alain G. Kapitho
%Date  : Jan. 2006

i = 1;
p(1) = p0;
tol = 1e-05;
while i <= N
   p(i+1) = g(p(i));
   if abs(p(i+1)-p(i)) < tol		%stopping criterion
      fprintf('The procedure was successful after k iterations %0.5g \n',rep);
%       disp('The root to the equation is')
      pnew = p(i+1);
      return
   end
   i = i+1;
end

if abs(p(i)-p(i-1)) > tol | i > N
   disp('The procedure was unsuccessful')
   disp('Condition |p(i+1)-p(i)| < tol was not sastified')
   tol
   disp('Please, examine the sequence of iterates')
   p = p'
   disp('In case you observe convergence, then increase the maximum number of iterations')
   disp('In case of divergence, try another initial approximation p0 or rewrite g(x)')
   disp('in such a way that |g''(x)|< 1 near the root')
   pnew=NaN;
end

%this part has to be changed accordingly with the specific function g(x)
function y = g(x)
global mu rho vu sigma sigmaw phie gamma psi delta
k1 = exp(x)/(1+exp(x));
theta= (1-gamma)/(1-1/psi);
A1 = (1-1/psi)/(1-k1*rho);
beta_ae = k1*A1*phie;
A2 = 0.5*((theta - theta/psi)^2+(theta*beta_ae)^2)/(theta*(1-k1*vu));
beta_aw = k1*A2;

A0 = (log(delta) + mu*(1-1/psi) + (log(1+exp(x))- k1*x) +  beta_aw * sigma^2 * (1-vu) + ... 
     0.5*theta* (beta_aw*sigmaw)^2) /(1-k1);

y=A0 + A2*sigma^2;




