%% Smulate a "typical disaster" [compare with Figure 7 in paper]
uu =0;               % upper bound for tracated norm Phi
ll    = -1000;      % lower bound for trancated norm Phi       
meanPhiStar  = meanPhi + ((normpdf((ll-meanPhi)/sigmaPhi) - normpdf((uu-meanPhi)/sigmaPhi))/(normcdf((uu-meanPhi)/sigmaPhi) - normcdf((ll-meanPhi)/sigmaPhi)))*sigmaPhi;
    
%  Initialize variables
II            =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
XX         =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
ZZ          =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
Eps        =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
CC         =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
Eta         =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
Phi         =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
Theta     =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);
Nu          =  zeros(nPrePeriods+nDisPeriods+nPostPeriods,1);

if onePerDis == 1
        ThetaDist = DistDropLongRun;
        PhiDist     = DistDropPeakTrough;
end

% Set initial values
XX(1)  = 1;
CC(1)  = XX(1) + ZZ(1) + Eps(1);
    
% Simulate Pre Period
for ii = 2:nPrePeriods
%     XX(ii)   = XX(ii-1) + muPre;
    XX(ii)   = XX(ii-1) + muPost;
    CC(ii)   = XX(ii) + ZZ(ii) + Eps(ii);
end
    
% Simulate Disaster Periods
for ii = nPrePeriods+1 : nPrePeriods+nDisPeriods
    II(ii)    = 1;
    if onePerDis == 0
        Theta(ii) = meanTheta;
    else
        Theta(ii) = mean(ThetaDist);
    end
    if onePerDis == 0
        Phi(ii)     = meanPhiStar;
    else
        Phi(ii)     = mean(PhiDist);
    end
    XX(ii)         = XX(ii-1) + muPost + Theta(ii)*II(ii);
%     XX(ii)         = XX(ii-1) + muMid + Theta(ii)*II(ii);
    ZZ(ii)         = rhoZZ*ZZ(ii-1) - Theta(ii)*II(ii) + Phi(ii)*II(ii);
    CC(ii)        = XX(ii) + ZZ(ii);
end

% Simulate Post Periods
for  ii  = nPrePeriods+nDisPeriods+1: nPrePeriods+nDisPeriods+nPostPeriods
    XX(ii)         = XX(ii-1) + muPost; 
    ZZ(ii)         = rhoZZ*ZZ(ii-1);
    CC(ii)        = XX(ii) + ZZ(ii);
end
% % plot(XX,'LineWidth', 2.5); hold on
% % plot(CC,'r','LineWidth', 2.5); legend ('potential consumption', 'consumption')
% % title('A Tipical Disaster')

detrendCC = detrend(CC,1);
detrendCC = detrendCC-detrendCC(nPrePeriods);

ccgrowth = trimr(CC,1,0)-trimr(CC,0,1);
gamma    = 4;

mu                     = muPost;
pWb                  = p_W;
pCb_NWD        = p_CbI;
pCe                   = p_Ce;
pCb_WD          =  p_CbW;
    
for ii = 1:size(stateGrid,1)
    EpsOnIntGrid    = intGrid(:,1);
    EtaOnIntGrid     = intGrid(:,2);
    % ThetaOnIntGrid = intGrid(:,3);
    PhiOnIntGrid     = intGrid(:,4);
    NuOnIntGrid      = intGrid(:,5);

    currentState = stateGrid(ii,:);
    curEps         = currentState(1);
    curZZ           = currentState(2);
    curII             =currentState(3);
    
    %% No Disasters case
    deltaLogC   = mu + EtaOnIntGrid + (rhoZZ - 1)*curZZ + NuOnIntGrid + (EpsOnIntGrid - curEps);
    intValNoDis = exp(-rhoU)*exp(deltaLogC).^(1-gamma);
    
    %% Disasters case
    deltaLogC   = mu + EtaOnIntGrid + (rhoZZ - 1)*curZZ + PhiOnIntGrid + NuOnIntGrid + (EpsOnIntGrid - curEps);
    intValDis      = exp(-rhoU)*exp(deltaLogC).^(1-gamma);
    
    %% Calculate probabilities
    pDis             = (1-curII) * (pWb*pCb_WD+(1-pWb)*pCb_NWD) + curII * (1-pCe);
    pNoDis        = 1 - pDis;
    
    %% Do the numerical integration
    PDivs(ii,1)   = weightsOverIntGrid * (intValNoDis * pNoDis + intValDis * pDis);
    PDivs(ii,1)   = log(PDivs(ii,1)); 
end

[EpsOnGrid, ZZOnGrid]     = SimOutputOnStateGrid(Eps, ZZ, DistMinZZ, DistMaxZZ);
[DisReturn, DisCGrowth]    = calcReturns(CC, EpsOnGrid, ZZOnGrid, II, PDivs, DistMinZZ, DistMaxZZ,0);

% Figure 7 in paper
figure
plot(DisReturn-1,'LineWidth',2.5); hold on
plot(detrendCC,'r','LineWidth',2.5); 
plot(zeros(40,1),':k', 'LineWidth',2.5); hold on
legend('bill returns','detrended consumption'); xlim([0 20]); %ylim([-0.4 0.4])
