function [DistDropBeginEnd,DistDropPeakTrough,DistDropLongRun,DistMaxZZ,DistMinZZ]...
         =DistDisasters_onePerPermDis(nSimDistDis,p_Ce,meanPhi,sigmaPhi,meanTheta,sigmaTheta)
% Calculate the distributions of ZZ, Peak to Trough and Long Run
%effects of the disaster. Used, e.g., in calcWeights for one period and permanent disasters.
rand('seed', 17617)
randn('seed', 17617)

IRlength = 35;  %IRlength is given in their paper - a long-term panel dataset for 35 countries

global rhoZZ

nSim = nSimDistDis;


DistDropBeginEnd   = zeros(nSim,1);
DistDropPeakTrough = zeros(nSim,1);
DistDropLongRun    = zeros(nSim,1);
DistMaxZZ          = zeros(nSim,1);
DistMinZZ          = zeros(nSim,1);

XX   = zeros(nSim, IRlength + 1);  % without eta and mu
ZZ   = zeros(nSim, IRlength + 1);  % without nu

for ii = 1:nSim
    phi     = zeros(IRlength, 1);
    theta   = zeros(IRlength, 1);
    
    disLength =    geornd(p_Ce) + 1;   % ppC[2] = 1-p_Ce; 1-ppC[2] = p_Ce
    if disLength > IRlength
        disLength = IRlength;
    end
    
    k = 1;
    while k >0 & k <=disLength
        rdnumber = meanPhi + sigmaPhi*randn(1,1);
        if rdnumber < 0 & rdnumber>-1000
            phi(k,1) = rdnumber; % truncated normal variable
            k = k +1;
        else
            k = k;
        end
    end
    theta(1:disLength,1) = meanTheta + sigmaTheta*randn(disLength,1);
    
    XX(ii,2)      = theta(1,1);
    ZZ(ii,2)      = phi(1,1) - theta(1,1);
    for jj = 3:IRlength + 1
        ZZ(ii,jj)   = (rhoZZ*ZZ(ii,jj-1) + phi(jj-1,1) - theta(jj-1,1));
        XX(ii,jj)   = XX(ii,jj-1)  + theta(jj-1,1);
    end
    
    DistDropBeginEnd(ii,1)     = XX(ii,disLength+1) + ZZ(ii,disLength+1);
    DistDropPeakTrough(ii,1)   = min(XX(ii,:) + ZZ(ii,:));
    DistDropLongRun(ii,1)      = XX(ii,IRlength+1) + ZZ(ii,IRlength+1);
    DistMaxZZ(ii,1)            = max(XX(ii,:));
    DistMinZZ(ii,1)            = min(XX(ii,:));
end


