function [weightsOverIntGrid] = calcWeights(intGridEps,  intGridEta, intGridTheta, intGridPhi, intGridNu,...
                                                                 tempGridEps, tempGridTheta, tempGridPhi, tempGridEta, tempGridNu,...
                                                                 DistDropLongRun, DistDropPeakTrough)

global onePerDis onePerPermDis nSdTheta nSdPhi
global muPost  sigmaEps sigmaEta  meanPhi  sigmaPhi  meanTheta sigmaTheta sigmaNu

fracCutOffTheta   = normcdf(-nSdTheta);
fracCutOffPhi     = normcdf(-nSdPhi);

nIntGridEps       = length(intGridEps);
nIntGridEta       = length(intGridEta);
nIntGridTheta     = length(intGridTheta);
nIntGridPhi       = length(intGridPhi);
nIntGridNu        = length(intGridNu);

nTempGridEps      = length(tempGridEps);
nTempGridEta      = length(tempGridEta);
nTempGridTheta    = length(tempGridTheta);
nTempGridPhi      = length(tempGridPhi);
nTempGridNu       = length(tempGridNu);
    
mu                = muPost;
    
%%  Weights over Eps
if nIntGridEps == 1
    wEps = 1;
else
    wEps = normcdf(tempGridEps(2:nTempGridEps)  , zeros(1, nTempGridEps-1), repmat(sigmaEps,1, nIntGridEps))-...
           normcdf(tempGridEps(1:nTempGridEps-1), zeros(1, nTempGridEps-1), repmat(sigmaEps,1,nIntGridEps));
    lowTailEps              = normcdf(tempGridEps(1), 0, sigmaEps);
    highTailEps             =1- normcdf(tempGridEps(nTempGridEps), 0, sigmaEps);
    wEps(1)                 = wEps(1) + lowTailEps;
    wEps(nIntGridEps)       = wEps(nIntGridEps) + highTailEps;
end

%%  Weights over Eta
if  nIntGridEta == 1
    wEta = 1;
else
    wEta = normcdf(tempGridEta(2:nTempGridEta)  , zeros(1,nTempGridEta-1), repmat(sigmaEta,1,nIntGridEta))- ...
           normcdf(tempGridEta(1:nTempGridEta-1), zeros(1,nTempGridEta-1), repmat(sigmaEta,1,nIntGridEta));
    lowTailEta             = normcdf(tempGridEta(1), 0, sigmaEta);
    highTailEta            =1- normcdf(tempGridEta(nTempGridEta), 0, sigmaEta);
    wEta(1)                = wEta(1) + lowTailEta;
    wEta(nIntGridEta)      = wEta(nIntGridEta) + highTailEta;
end

%%  Weights over Theta
if nIntGridTheta == 1
    wTheta = 1;
else
    if onePerDis==0 & onePerPermDis==0
        wTheta = normcdf(tempGridTheta(2:nTempGridTheta)  , repmat(meanTheta,1,nIntGridTheta), repmat(sigmaTheta,1,nIntGridTheta))-...
                 normcdf(tempGridTheta(1:nTempGridTheta-1), repmat(meanTheta,1,nIntGridTheta), repmat(sigmaTheta,1,nIntGridTheta));
        lowTailTheta                 = normcdf(tempGridTheta(1), meanTheta, sigmaTheta);
        highTailTheta                = 1- normcdf(tempGridTheta(nTempGridTheta), meanTheta, sigmaTheta);
        wTheta(1)                    = wTheta(1) + lowTailTheta;
        wTheta(nIntGridTheta) = wTheta(nIntGridTheta) + highTailTheta;
    elseif onePerDis ~=0
        %Weights if we use getSimpleSequence to get grid     
        wTheta = WeightsSimpleSequence(DistDropLongRun, tempGridTheta);
    elseif onePerPermDis~=0
        wTheta = WeightsSimpleSequence(DistDropPeakTrough, tempGridTheta);
        % Weights if we use getDistQuantiles to get grid
        % wTheta    <- numeric(nIntGridTheta) + (1-2*fracCutOffTheta)/nIntGridTheta
        % wTheta[1] <- wTheta[1] + fracCutOffTheta
        % wTheta[nIntGridTheta] <- wTheta[nIntGridTheta] + fracCutOffTheta
    end
end

%% Weights over Phi
if nIntGridPhi == 1
    wPhi = 1;
else
%     if onePerDis==0 & onePerPermDis==0
        temp1 = normcdf(tempGridPhi(2:nTempGridPhi), repmat(meanPhi,1,nIntGridPhi ), repmat(sigmaPhi,1,nIntGridPhi))-...
                normcdf(repmat(-1000,1,nIntGridPhi ), repmat(meanPhi,1,nIntGridPhi), repmat(sigmaPhi,1,nIntGridPhi));
        temp2 = normcdf(repmat(0,1,nIntGridPhi), repmat(meanPhi,1,nIntGridPhi), repmat(sigmaPhi,1,nIntGridPhi))-...
                normcdf(repmat(-1000,1,nIntGridPhi), repmat(meanPhi,1,nIntGridPhi), repmat(sigmaPhi,1,nIntGridPhi));
        temp3 = normcdf(tempGridPhi(1:nTempGridPhi-1), repmat(meanPhi,1,nIntGridPhi), repmat(sigmaPhi,1,nIntGridPhi))-...
            normcdf(repmat(-1000,1,nIntGridPhi), repmat(meanPhi,1,nIntGridPhi), repmat(sigmaPhi,1,nIntGridPhi));
        wPhi   =  temp1./temp2 - temp3./temp2;
        lowTailPhi        =  (normcdf(tempGridPhi(1), meanPhi, sigmaPhi)-normcdf(-1000, meanPhi, sigmaPhi))/...
            (normcdf(0, meanPhi, sigmaPhi)-normcdf(-1000, meanPhi, sigmaPhi))  ;
        highTailPhi       =  1- (normcdf(tempGridPhi(nTempGridPhi), meanPhi, sigmaPhi)-normcdf(-1000, meanPhi, sigmaPhi))/...
            (normcdf(0, meanPhi, sigmaPhi)-normcdf(-1000, meanPhi, sigmaPhi));
        wPhi(1)                  = wPhi(1) + lowTailPhi;
        wPhi(nIntGridPhi) = wPhi(nIntGridPhi) + highTailPhi;
%     elseif onePerDis ~=0
%         %Weights if we use getSimpleSequence to get grid
%         
%         wPhi = WeightsSimpleSequence(DistDropPeakTrough, tempGridPhi);
%     elseif onePerPermDis ~=0
%         wPhi = WeightsSimpleSequence(DistDropPeakTrough, tempGridPhi);
%         % Weights if we use getDistQuantiles to get grid
%         % wPhi    <- numeric(nIntGridPhi) + (1-2*fracCutOffPhi)/nIntGridPhi
%         % wPhi[1] <- wPhi[1] + fracCutOffPhi
%         % wPhi[nIntGridPhi] <- wPhi[nIntGridPhi] + fracCutOffPhi
%     end
end

%%  Weights over Nu
if nIntGridNu == 1
    wNu = 1;
else
    wNu = normcdf(tempGridNu(2:nTempGridNu)  , zeros(1,nTempGridNu-1), repmat(sigmaNu,1,nIntGridNu))-...
          normcdf(tempGridNu(1:nTempGridNu-1), zeros(1,nTempGridNu-1), repmat(sigmaNu,1,nIntGridNu));
    lowTailNu            = normcdf(tempGridNu(1), 0, sigmaNu);
    highTailNu           = 1- normcdf(tempGridNu(nTempGridNu), 0, sigmaNu);
    wNu(1)               = wNu(1) + lowTailNu;
    wNu(nIntGridNu)      = wNu(nIntGridNu) + highTailNu;
end

%% Create weights for all states
% Kronecker product
weightsOverIntGrid = kron(wEta, wEps);
weightsOverIntGrid = kron(wTheta, weightsOverIntGrid);
weightsOverIntGrid = kron(wPhi, weightsOverIntGrid);
weightsOverIntGrid = kron(wNu, weightsOverIntGrid);




