function nextStateFuncGen (stateGrid, stateGridEps, stateGridZZ, intGrid, ...
    intGridEps, intGridEta, intGridTheta, intGridPhi, intGridNu,simulation)

% simulation = 1;  yields the results in FOTY paper
% simulation = 0;  yields the replication results for NUSB's paper

global rhoZZ onePerPermDis MultiPerPermDis NoShortRunShock

nIntGridEps    = length(intGridEps);
nIntGridEta    = length(intGridEta);
nIntGridTheta  = length(intGridTheta);
nIntGridPhi    = length(intGridPhi);
nIntGridNu     = length(intGridNu);

nStateGridEps = length(stateGridEps);
nStateGridZZ  = length(stateGridZZ);

nStateGrid    = size(stateGrid,1);
nInt          = size(intGrid,1);

for ii = 1:nStateGrid
    fprintf('NextStateGen at state %4.0f \n',ii);
    
    oldZZ          = stateGrid(ii,2);
    
    %%  No-disaster case
    nextStateNoDis = zeros(1,nInt);
    for iiNu = 1:nIntGridNu
        runZZ       = rhoZZ*oldZZ + intGridNu(iiNu);
        temp        = abs(stateGridZZ-runZZ);
        indRunZZ= find(temp == min(temp)); clear temp
        indRunZZ= indRunZZ(1);          % if there is more than one obs equal to the min, we always pick up the first index
        
        % We can skip Phi, Theta and Eta since ZZ and Eps are constant for these in this case
        nextStateEps   = zeros(1, nIntGridEps);
        for iiEps = 1:nIntGridEps
            runEps          = intGridEps(iiEps);
            temp            = abs(stateGridEps-runEps);
            indRunEps       = find(temp == min(temp)); clear temp
            nextStateEps(iiEps) = (indRunZZ-1)*nStateGridEps + indRunEps;
        end
        nextStateEps   = repmat(nextStateEps,1, nIntGridEta*nIntGridTheta*nIntGridPhi);
        startInd       = (iiNu-1)*nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi + 1;
        endInd         =  iiNu   *nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi;
        nextStateNoDis(startInd:endInd) = nextStateEps;
    end
    
    %% Disaster case
    nextStateDis = zeros(1, nInt);
    for iiNu  = 1:nIntGridNu
        for iiPhi = 1:nIntGridPhi
            for iiTheta = 1:nIntGridTheta
                runZZ       = rhoZZ*oldZZ - intGridTheta(iiTheta) + intGridPhi(iiPhi) + intGridNu(iiNu);
                temp        = abs(stateGridZZ-runZZ);
                indRunZZ    = find(temp == min(temp)); clear temp
                
                % We can skip Eta since ZZ and Eps are constant over Eta
                nextStateEps    = zeros(1,nIntGridEps);
                for iiEps = 1:nIntGridEps
                    runEps    =  intGridEps(iiEps);
                    temp      = abs(stateGridEps-runEps);
                    indRunEps = find(temp == min(temp)); clear temp
                    nextStateEps(iiEps) = nStateGridZZ*nStateGridEps + (indRunZZ-1)*nStateGridEps + indRunEps;
                end
                nextStateEps   = repmat(nextStateEps,1, nIntGridEta);
                
                startInd      = (iiNu-1)*nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi+...
                    (iiPhi-1)*nIntGridEps*nIntGridEta*nIntGridTheta+...
                    (iiTheta-1)*nIntGridEps*nIntGridEta + 1;
                endInd        = (iiNu-1)*nIntGridEps*nIntGridEta*nIntGridTheta*nIntGridPhi+...
                    (iiPhi-1)*nIntGridEps*nIntGridEta*nIntGridTheta+...
                    (iiTheta-1)*nIntGridEps*nIntGridEta + length(nextStateEps);
                
                nextStateDis(startInd:endInd) = nextStateEps;
                
                %cat('[iiNu, iiPhi, iiTheta] = ',iiNu,' ',iiPhi,' ',iiTheta,'\n')
                %print(nextStateEps)
                %cat('\n')
                %browser()
            end
        end
    end
    
    if simulation == 0
        if NoShortRunShock == 1
            fname = [cd, '\Results\replication\RepNSBU\nextState_NoShortRunShock_',num2str(ii)];
        elseif onePerPermDis == 0 & MultiPerPermDis == 0
            fname = [cd, '\Results\replication\RepNSBU\nextState_',num2str(ii)];
        elseif onePerPermDis == 1 & MultiPerPermDis == 0
            fname = [cd, '\Results\replication\RepNSBU\nextState_onePerPermDis',num2str(ii)];
        elseif onePerPermDis == 0 & MultiPerPermDis == 1
            fname = [cd, '\Results\replication\RepNSBU\nextState_MultiPermDis',num2str(ii)];
        end
        save(fname,'nextStateDis','nextStateNoDis')
    else
        if onePerPermDis == 0 
            fname = [cd, '\Results\simulation\SimNSBU\nextState_',num2str(ii)];
        elseif onePerPermDis == 1 
            fname = [cd, '\Results\simulation\SimNSBU\nextState__onePerPermDis',num2str(ii)];
        end
        save(fname,'nextStateDis','nextStateNoDis')
    end
end



