%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot of all the simulation results
% Figure 1,2,3 reported in the paper
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; clc; %close all;
addpath('.\Results\simulation','.\Results\HJBounds', '.\Library')
%% 
SET = 5;               % 1: SET A: stock market + risk free bond + 5-year government bonds
                       % 5: SET E: new SET B in the revised version of JBES, SET A + value-middle-growth portfolios
preference = 0; % 0: sensitivity without preference parameters
                % 1: sensitivity with all paramters   

SDF_MH=[cd,'\Results\HJBounds\SDF_MH_SET',num2str(SET)];

load(SDF_MH)


%%  Plot of all the simulation results 
NSIM = 10;
  

% HJB_bootstrapIndex = [cd,'\Results\HJBounds\BL_bootstrapIndex_SET',num2str(SET),'_CL90'];
% load(HJB_bootstrapIndex)
% CI_L    = [CI_BL_1Q(:,1) CI_BL_1Y(:,1) CI_BL_5Y(:,1)]';
% CI_U    = [CI_BL_1Q(:,2) CI_BL_1Y(:,2) CI_BL_5Y(:,2)]';
% 
%% Figure 1: Bounds v.s. Models
sim_LRR_SSY_T600000=[cd, '\Results\simulation\sim_LRRSSY_NSIM10_T600000_SET1'];
load(sim_LRR_SSY_T600000)
SSY_long_mean = [mean(simmean_1Q(2:10,1)) mean(simmean_1Y(2:10,1)) mean(simmean_5Y(2:10,1))];
SSY_long_std = [mean(simsd_1Q(2:10,1)) mean(simsd_1Y(2:10,1)) mean(simsd_5Y(2:10,1))];
clear simmean_1Q simsd_1Q simmean_1Y simsd_1Y simmean_5Y simsd_5Y 

sim_LRRBKY2016_NSIM10_T600000=[cd, '\Results\simulation\sim_LRRBKY2016_NSIM10_T600000'];
load(sim_LRRBKY2016_NSIM10_T600000)
BKY_long_mean = [mean(simmean_c([1:5 7:10],1)) mean(simmean_c([1:5 7:10],2)) mean(simmean_c([1:5 7:10],3))];
BKY_long_std = [mean(simsd_c([1:5 7:10],1)) mean(simsd_c([1:5 7:10],2)) mean(simsd_c([1:5 7:10],3))];

sim_NSBU2013_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NSIM10_T150000'];
load(sim_NSBU2013_NSIM10_T150000)
RD_long_mean = [mean(simmean_c(1:9,1)) mean(simmean_c(1:9,2)) ];
RD_long_std = [mean(simsd_c(1:9,1)) mean(simsd_c(1:9,2))];

sim_NSBU2013_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NoDisasters_NSIM10_T150000'];
load(sim_NSBU2013_NSIM10_T150000)
RD_NoDisasters_long_mean = [mean(simmean_c(1:9,1)) mean(simmean_c(1:9,2)) ];
RD_NoDisasters_long_std = [mean(simsd_c(1:9,1)) mean(simsd_c(1:9,2))];

figure
for h =1:size(BKY_long_mean,2)
    hold off
    subplot(1,3,h); 
    plot(E_m, sqrt(sig2_BL(h,:)),'k','LineWidth',4); hold on
    plot(E_m, sqrt(sig2_HJ(h,:)),'Color',[0.48 0.06 0.89],'LineStyle','--','LineWidth',4.5); hold on
    if h == 1
        xlim([0.98 1.01]); title('1-Quarter')
        ylim([0 1.6])
    elseif h ==2
        xlim([0.8 1.025]); title('1-Year')
         ylim([0 2.5])
    elseif h ==3
        xlim([0.3 1.28]); title('5-Year')
         ylim([0 6.5])
    end
    plot(mean(BKY_long_mean(:,h)),mean(BKY_long_std (:,h)),'^r','MarkerSize',14,'MarkerFaceColor','r');hold on
    plot(mean(SSY_long_mean(:,h)),mean(SSY_long_std (:,h)),'pb','MarkerSize',19,'MarkerFaceColor','b');hold on
    if h ~=1
    plot(mean(RD_long_mean(:,h-1)),mean(RD_long_std (:,h-1)),'og','MarkerSize',19,'MarkerFaceColor','g');hold on
    plot(mean(RD_NoDisasters_long_mean(:,h-1)),mean(RD_NoDisasters_long_std (:,h-1)),'sk','MarkerSize',19,'MarkerFaceColor','k');hold on 
    end
    xlabel('E(m)')
    ylabel('standard deviation')
end
% h_legend = legend('predictors-based bound','HJ bound','simulated value with estimated parameters','1-sd confidence area',...
%                  'simulated value with calibrated parameters');
h_legend = legend('predictors-based bound','HJ bound','simulated value -- BKY',...
                 'simulated value -- SSY','simulated value -- NSBU baseline','simulated value -- NSBU NoDisasters');
h_legend = legend('Orientation','horizontal','predictors-based bound','HJ bound');
set(h_legend,'FontSize',13);


%% Rare Disaster by Nakamura et al. (2013)

%% Figure 2: comparison with the case with one period permanent disaster and multiple permanent disasters, Table 7 NSBU, case 1,2,3
sim_NSBU2013_MultiPerPermDis_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_MultiPerPermDis_NSIM10_T150000'];
load(sim_NSBU2013_MultiPerPermDis_NSIM10_T150000)
mean_1Y_MultiPer=simmean_c(:,1);  sd_1Y_MultiPer=simsd_c(:,1);
mean_5Y_MultiPer=simmean_c(:,2);  sd_5Y_MultiPer=simsd_c(:,2);

sim_NSBU2013_onePerPermDis_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_onePerPermDis_NSIM10_T150000'];
load(sim_NSBU2013_onePerPermDis_NSIM10_T150000)
mean_1Y_onePer=simmean_c(:,1);  sd_1Y_onePer=simsd_c(:,1);
mean_5Y_onePer=simmean_c(:,2);  sd_5Y_onePer=simsd_c(:,2);

sim_NSBU2013_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NSIM10_T150000'];
load(sim_NSBU2013_NSIM10_T150000)

figure
for h =1:2
    hold off
    subplot(1,2,h); 
    plot(E_m, sqrt(sig2_BL(h+1,:)),'k','LineWidth',4); hold on
    plot(E_m, sqrt(sig2_HJ(h+1,:)),'Color',[0.48 0.06 0.89],'LineStyle','--','LineWidth',4.5); hold on
    if h == 1
        xlim([0.95 1.02]); title('1-Year')
        ylim([0.5 2.5])
%         ylim([0.2 1.5]) %ylim([0 1.5])
    elseif h ==2
        xlim([0.65 1.25]); title('5-Year')
        ylim([0.5 8])%ylim([0 4])
    end
    x0 = mean(simmean_c(1:9,h)); y0 = mean(simsd_c(1:9,h)); % Center of ellip
    plot(x0,y0,'^r','MarkerSize',15,'MarkerFaceColor','r'); hold on
    if h == 1
        plot(mean(mean_1Y_onePer),mean(sd_1Y_onePer),'^b','MarkerSize',15,'MarkerFaceColor','b');hold on
        plot(mean(mean_1Y_MultiPer),mean(sd_1Y_MultiPer),'^k','MarkerSize',15,'MarkerFaceColor','k');hold on
    else
         plot(mean(mean_5Y_onePer(1:9)),mean(sd_5Y_onePer(1:9)),'^b','MarkerSize',15,'MarkerFaceColor','b');hold on
         plot(mean(mean_5Y_MultiPer(1:9)),mean(sd_5Y_MultiPer(1:9)),'^k','MarkerSize',15,'MarkerFaceColor','k');hold on
    end
    xlabel('E(m)')
    ylabel('standard deviation')
end
h_legend = legend('predictors-based bound','HJ bound','simulated value - Baseline','simulated value - Permanent and one period','simulated value - Permanent');
h_legend = legend('Orientation','horizontal','predictors-based bound','HJ bound');
set(h_legend,'FontSize',13);



%% Figure 3: sensitivity of preference, Table 7 NSBU, case 1,5,6; and SSY with different gamma

sim_LRR_SSY_T600000=[cd, '\Results\simulation\sim_LRRSSY_NSIM10_T600000_SET1'];
load(sim_LRR_SSY_T600000)
SSY_long_mean = [mean(simmean_1Q(2:10,1)) mean(simmean_1Y(2:10,1)) mean(simmean_5Y(2:10,1))];
SSY_long_std = [mean(simsd_1Q(2:10,1)) mean(simsd_1Y(2:10,1)) mean(simsd_5Y(2:10,1))];
clear simmean_1Q simsd_1Q simmean_1Y simsd_1Y simmean_5Y simsd_5Y 

sim_LRR_SSY_T600000=[cd, '\Results\simulation\sim_LRRSSY_NSIM10_T600000_LowGamma'];
load(sim_LRR_SSY_T600000)
SSY_long_mean_LowGamma = [mean(simmean_1Q(2:10,1)) mean(simmean_1Y(2:10,1)) mean(simmean_5Y(2:10,1))];
SSY_long_std_LowGamma = [mean(simsd_1Q(2:10,1)) mean(simsd_1Y(2:10,1)) mean(simsd_5Y(2:10,1))];
clear simmean_1Q simsd_1Q simmean_1Y simsd_1Y simmean_5Y simsd_5Y 

sim_LRR_SSY_T600000=[cd, '\Results\simulation\sim_LRRSSY_NSIM10_T600000_HighGamma'];
load(sim_LRR_SSY_T600000)
SSY_long_mean_HighGamma = [mean(simmean_1Q(2:10,1)) mean(simmean_1Y(2:10,1)) mean(simmean_5Y(2:10,1))];
SSY_long_std_HighGamma = [mean(simsd_1Q(2:10,1)) mean(simsd_1Y(2:10,1)) mean(simsd_5Y(2:10,1))];
clear simmean_1Q simsd_1Q simmean_1Y simsd_1Y simmean_5Y simsd_5Y

sim_NSBU2013_LowGamma_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_LowGamma_NSIM10_T150000'];
load(sim_NSBU2013_LowGamma_NSIM10_T150000)
mean_1Y_LowGamma=simmean_c(:,1);
mean_5Y_LowGamma=simmean_c(:,2);
sd_1Y_LowGamma=simsd_c(:,1);
sd_5Y_LowGamma=simsd_c(:,2);

sim_NSBU2013_HighGammas_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_HighGamma_NSIM10_T150000'];
load(sim_NSBU2013_HighGammas_NSIM10_T150000)
mean_1Y_HighGamma=simmean_c(:,1);
mean_5Y_HighGamma=simmean_c(:,2);
sd_1Y_HighGamma=simsd_c(:,1);
sd_5Y_HighGamma=simsd_c(:,2);

sim_NSBU2013_NSIM10_T150000=[cd, '\Results\simulation\SimNSBU\sim_NSBU2013_NSIM10_T150000'];
load(sim_NSBU2013_NSIM10_T150000)

figure
for h =1:3
    hold off
    subplot(1,3,h); 
    plot(E_m, sqrt(sig2_BL(h,:)),'k','LineWidth',4); hold on
    plot(E_m, sqrt(sig2_HJ(h,:)),'Color',[0.48 0.06 0.89],'LineStyle','--','LineWidth',4.5); hold on
    if h == 1
        xlim([0.95 1.02]); title('1-Year')
        ylim([0 2.5])
    elseif h == 2
        xlim([0.8 1.152]); title('1-Year')
        ylim([0 6])
        %         ylim([0.2 1.5]) %ylim([0 1.5])
    elseif h ==3
        xlim([0.35 1.5]); title('5-Year')
        ylim([0.5 18])%ylim([0 4])
    end
    plot(mean(SSY_long_mean(:,h)),mean(SSY_long_std (:,h)),'pb','MarkerSize',19,'MarkerFaceColor','b');hold on
    plot(mean(SSY_long_mean_LowGamma(:,h)),mean(SSY_long_std_LowGamma (:,h)),'pb','MarkerSize',19,'MarkerFaceColor','b');hold on
    plot(mean(SSY_long_mean_HighGamma(:,h)),mean(SSY_long_std_HighGamma(:,h)),'pb','MarkerSize',19,'MarkerFaceColor','b');hold on
    
   
    if h == 2
        x0 = mean(simmean_c(1:9,h-1)); y0 = mean(simsd_c(1:9,h-1)); % Center of ellip
        plot(x0,y0,'^r','MarkerSize',12,'MarkerFaceColor','r'); hold on
        plot(mean(mean_1Y_LowGamma),mean(sd_1Y_LowGamma),'^r','MarkerSize',15,'MarkerFaceColor','r');hold on
        plot(mean(mean_1Y_HighGamma),mean(sd_1Y_HighGamma),'^r','MarkerSize',15,'MarkerFaceColor','r');hold on
    elseif h == 3
         x0 = mean(simmean_c(1:9,h-1)); y0 = mean(simsd_c(1:9,h-1)); % Center of ellip
         plot(x0,y0,'^r','MarkerSize',12,'MarkerFaceColor','r'); hold on
         plot(mean(mean_5Y_LowGamma(1:9)),mean(sd_5Y_LowGamma(1:9)),'^r','MarkerSize',15,'MarkerFaceColor','r');hold on
         plot(mean(mean_5Y_HighGamma(1:9)),mean(sd_5Y_HighGamma(1:9)),'^r','MarkerSize',15,'MarkerFaceColor','r');hold on
    end
    xlabel('E(m)')
    ylabel('standard deviation')
end
% h_legend = legend('predictors-based bound','HJ bound','simulated value - Baseline','simulated value - Permanent and one period','1-sd confidence area');
h_legend = legend('predictors-based bound','HJ bound','simulated value - Baseline','simulated value - Low Gamma','simulated value - High Gamma');
% h_legend = legend('Orientation','horizontal','predictors-based bound','HJ bound');
set(h_legend,'FontSize',13);


